@@ -1,10 +1,10 @@
-/**
- * XMLSec library
+/*
+ * XML Security Library (http://www.aleksey.com/xmlsec).
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #ifndef __XMLSEC_OPENSSL_SYMBOLS_H__
 #define __XMLSEC_OPENSSL_SYMBOLS_H__
@@ -37,6 +37,7 @@ extern "C" {
 #define xmlSecKeyDataAesId                      xmlSecOpenSSLKeyDataAesId
 #define xmlSecKeyDataDesId                      xmlSecOpenSSLKeyDataDesId
 #define xmlSecKeyDataDsaId                      xmlSecOpenSSLKeyDataDsaId
+#define xmlSecKeyDataEcdsaId                    xmlSecOpenSSLKeyDataEcdsaId
 #define xmlSecKeyDataHmacId                     xmlSecOpenSSLKeyDataHmacId
 #define xmlSecKeyDataRsaId                      xmlSecOpenSSLKeyDataRsaId
 #define xmlSecKeyDataX509Id                     xmlSecOpenSSLKeyDataX509Id
@@ -57,12 +58,21 @@ extern "C" {
 #define xmlSecTransformAes128CbcId              xmlSecOpenSSLTransformAes128CbcId
 #define xmlSecTransformAes192CbcId              xmlSecOpenSSLTransformAes192CbcId
 #define xmlSecTransformAes256CbcId              xmlSecOpenSSLTransformAes256CbcId
+#define xmlSecTransformAes128GcmId              xmlSecOpenSSLTransformAes128GcmId
+#define xmlSecTransformAes192GcmId              xmlSecOpenSSLTransformAes192GcmId
+#define xmlSecTransformAes256GcmId              xmlSecOpenSSLTransformAes256GcmId
 #define xmlSecTransformKWAes128Id               xmlSecOpenSSLTransformKWAes128Id
 #define xmlSecTransformKWAes192Id               xmlSecOpenSSLTransformKWAes192Id
 #define xmlSecTransformKWAes256Id               xmlSecOpenSSLTransformKWAes256Id
 #define xmlSecTransformDes3CbcId                xmlSecOpenSSLTransformDes3CbcId
 #define xmlSecTransformKWDes3Id                 xmlSecOpenSSLTransformKWDes3Id
 #define xmlSecTransformDsaSha1Id                xmlSecOpenSSLTransformDsaSha1Id
+#define xmlSecTransformDsaSha256Id              xmlSecOpenSSLTransformDsaSha256Id
+#define xmlSecTransformEcdsaSha1Id              xmlSecOpenSSLTransformEcdsaSha1Id
+#define xmlSecTransformEcdsaSha224Id            xmlSecOpenSSLTransformEcdsaSha224Id
+#define xmlSecTransformEcdsaSha256Id            xmlSecOpenSSLTransformEcdsaSha256Id
+#define xmlSecTransformEcdsaSha384Id            xmlSecOpenSSLTransformEcdsaSha384Id
+#define xmlSecTransformEcdsaSha512Id            xmlSecOpenSSLTransformEcdsaSha512Id
 #define xmlSecTransformHmacMd5Id                xmlSecOpenSSLTransformHmacMd5Id
 #define xmlSecTransformHmacRipemd160Id          xmlSecOpenSSLTransformHmacRipemd160Id
 #define xmlSecTransformHmacSha1Id               xmlSecOpenSSLTransformHmacSha1Id
@@ -86,6 +96,9 @@ extern "C" {
 #define xmlSecTransformSha256Id                 xmlSecOpenSSLTransformSha256Id
 #define xmlSecTransformSha384Id                 xmlSecOpenSSLTransformSha384Id
 #define xmlSecTransformSha512Id                 xmlSecOpenSSLTransformSha512Id
+#define xmlSecTransformGost2001GostR3411_94Id   xmlSecOpenSSLTransformGost2001GostR3411_94Id
+#define xmlSecTransformGostR3411_94Id           xmlSecOpenSSLTransformGostR3411_94Id
+
 
 /********************************************************************
  *
