@@ -1,4 +1,4 @@
-/**
+/*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
  * The transforms engine
@@ -6,24 +6,29 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #ifndef __XMLSEC_TRANSFORMS_H__
 #define __XMLSEC_TRANSFORMS_H__
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
 
+#include <xmlsec/exports.h>
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/buffer.h>
 #include <xmlsec/list.h>
 #include <xmlsec/nodeset.h>
 #include <xmlsec/keys.h>
 
+#ifndef XMLSEC_NO_XSLT
+#include <libxslt/security.h>
+#endif /* XMLSEC_NO_XSLT */
+
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 typedef const struct _xmlSecTransformKlass              xmlSecTransformKlass,
                                                         *xmlSecTransformId;
 
@@ -33,7 +38,7 @@ typedef const struct _xmlSecTransformKlass              xmlSecTransformKlass,
  * The binary data chunks size. XMLSec processes binary data one chunk
  * at a time. Changing this impacts xmlsec memory usage and performance.
  */
-#define XMLSEC_TRANSFORM_BINARY_CHUNK                   64
+#define XMLSEC_TRANSFORM_BINARY_CHUNK                   1024
 
 /**********************************************************************
  *
@@ -129,7 +134,7 @@ typedef unsigned int                            xmlSecTransformUriType;
 /**
  * xmlSecTransformUriTypeSameDocument:
  *
- * The smae document ("#...") but not empty ("") URI type.
+ * The same document ("#...") but not empty ("") URI type.
  */
 #define xmlSecTransformUriTypeSameDocument      0x0002
 
@@ -296,7 +301,7 @@ typedef int             (*xmlSecTransformCtxPreExecuteCallback)         (xmlSecT
  *                      additional validation (and abort transform execution
  *                      if needed).
  * @result:             the pointer to transforms result buffer.
- * @status:             the transforms chain processng status.
+ * @status:             the transforms chain processing status.
  * @uri:                the data source URI without xpointer expression.
  * @xptrExpr:           the xpointer expression from data source URI (if any).
  * @first:              the first transform in the chain.
@@ -376,7 +381,7 @@ XMLSEC_EXPORT void                      xmlSecTransformCtxDebugXmlDump  (xmlSecT
 /**
  * xmlSecTransform:
  * @id:                 the transform id (pointer to #xmlSecTransformId).
- * @operation:          the transform's opertaion.
+ * @operation:          the transform's operation.
  * @status:             the current status.
  * @hereNode:           the pointer to transform's <dsig:Transform /> node.
  * @next:               the pointer to next transform in the chain.
@@ -748,7 +753,7 @@ typedef int             (*xmlSecTransformExecuteMethod)         (xmlSecTransform
  * @href:                       the transform's identification string (href).
  * @usage:                      the allowed transforms usages.
  * @initialize:                 the initialization method.
- * @finalize:                   the finmalization (destroy) function.
+ * @finalize:                   the finalization (destroy) function.
  * @readNode:                   the XML node read method.
  * @writeNode:                  the XML node write method.
  * @setKeyReq:                  the set key requirements method.
@@ -764,7 +769,7 @@ typedef int             (*xmlSecTransformExecuteMethod)         (xmlSecTransform
  * @reserved0:                  reserved for the future.
  * @reserved1:                  reserved for the future.
  *
- * The transform klass desccription structure.
+ * The transform klass description structure.
  */
 struct _xmlSecTransformKlass {
     /* data */
@@ -801,7 +806,7 @@ struct _xmlSecTransformKlass {
 
 /**
  * xmlSecTransformKlassGetName:
- * @klass:              the transofrm's klass.
+ * @klass:              the transform's klass.
  *
  * Macro. Returns transform klass name.
  */
@@ -949,8 +954,16 @@ XMLSEC_EXPORT int               xmlSecTransformXPointerSetExpr          (xmlSecT
                                                                          const xmlChar* expr,
                                                                          xmlSecNodeSetType nodeSetType,
                                                                          xmlNodePtr hereNode);
+/**
+ * xmlSecTransformRelationshipId:
+ *
+ * The Relationship transform klass.
+ */
+#define xmlSecTransformRelationshipId \
+        xmlSecTransformRelationshipGetKlass()
+XMLSEC_EXPORT xmlSecTransformId xmlSecTransformRelationshipGetKlass     (void);
+
 #ifndef XMLSEC_NO_XSLT
-#include <libxslt/security.h>
 
 /**
  * xmlSecTransformXsltId:
@@ -986,6 +999,19 @@ XMLSEC_EXPORT xmlSecTransformId xmlSecTransformVisa3DHackGetKlass       (void);
 XMLSEC_EXPORT int               xmlSecTransformVisa3DHackSetID          (xmlSecTransformPtr transform,
                                                                          const xmlChar* id);
 
+
+
+/*********************************************************************
+ *
+ * Helper transform functions
+ *
+ ********************************************************************/
+
+#ifndef XMLSEC_NO_HMAC
+XMLSEC_EXPORT xmlSecSize        xmlSecTransformHmacGetMinOutputBitsSize(void);
+XMLSEC_EXPORT void              xmlSecTransformHmacSetMinOutputBitsSize(xmlSecSize val);
+#endif /* XMLSEC_NO_HMAC */
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
