@@ -6,7 +6,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 
 #ifndef __XMLSEC_GLOBALS_H__
@@ -22,4 +22,7 @@
 #define IN_XMLSEC
 #define XMLSEC_PRIVATE
 
+/* Include common error helper macros. */
+#include "errors_helpers.h"
+
 #endif /* __XMLSEC_GLOBALS_H__ */
