@@ -1,14 +1,20 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
- * X509 support
  *
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2010-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
+/**
+ * SECTION:x509utils
+ * @Short_description: X509 certificates support functions for GnuTLS.
+ * @Stability: Private
+ *
+ */
+
 #include "globals.h"
 
 #ifndef XMLSEC_NO_X509
@@ -20,16 +26,11 @@
 #include <errno.h>
 #include <time.h>
 
-#include <libxml/tree.h>
-
-
-
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 #include <gnutls/pkcs12.h>
 
 #include <xmlsec/xmlsec.h>
-#include <xmlsec/xmltree.h>
 #include <xmlsec/keys.h>
 #include <xmlsec/keyinfo.h>
 #include <xmlsec/keysmngr.h>
@@ -42,7 +43,7 @@
 #include <xmlsec/gnutls/x509.h>
 
 #include "x509utils.h"
-
+#include "../cast_helpers.h"
 
 /**************************************************************************
  *
@@ -162,30 +163,39 @@ xmlSecGnuTLSX509CrlListDebugXmlDumpItem(xmlSecPtr ptr, FILE* output) {
  *
  ************************************************************************/
 
-/* HACK: gnutls doesn't have cert duplicate function, so we simply 
+/* HACK: gnutls doesn't have cert duplicate function, so we simply
  write cert out and then read it back */
 gnutls_x509_crt_t
 xmlSecGnuTLSX509CertDup(gnutls_x509_crt_t src) {
-    xmlChar * buf = NULL;
+    xmlSecBuffer buf;
     gnutls_x509_crt_t res = NULL;
+    int ret;
 
     xmlSecAssert2(src != NULL, NULL);
 
-    buf = xmlSecGnuTLSX509CertBase64DerWrite(src, 0);
-    if(buf == NULL) {
-        xmlSecInternalError("xmlSecGnuTLSX509CertBase64DerWrite", NULL);
+    ret = xmlSecBufferInitialize(&buf, 0);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecBufferInitialize", NULL);
         return (NULL);
     }
 
-    res = xmlSecGnuTLSX509CertBase64DerRead(buf);
+    ret = xmlSecGnuTLSX509CertDerWrite(src, &buf);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGnuTLSX509CertDerWrite", NULL);
+        xmlSecBufferFinalize(&buf);
+        return (NULL);
+    }
+
+    res = xmlSecGnuTLSX509CertRead(xmlSecBufferGetData(&buf), xmlSecBufferGetSize(&buf),
+        xmlSecKeyDataFormatCertDer);
     if(res == NULL) {
-        xmlSecInternalError("xmlSecGnuTLSX509CertBase64DerRead", NULL);
-        xmlFree(buf);
+        xmlSecInternalError("xmlSecGnuTLSX509CertRead", NULL);
+        xmlSecBufferFinalize(&buf);
         return (NULL);
     }
 
     /* done */
-    xmlFree(buf);
+    xmlSecBufferFinalize(&buf);
     return (res);
 }
 
@@ -301,73 +311,12 @@ xmlSecGnuTLSX509CertGetIssuerSerial(gnutls_x509_crt_t cert) {
     return(res);
 }
 
-xmlChar *
-xmlSecGnuTLSX509CertGetSKI(gnutls_x509_crt_t cert) {
-    xmlChar * res = NULL;
-    xmlSecByte* buf = NULL;
-    size_t bufSize = 0;
-    unsigned int critical = 0;
-    int err;
-
-    xmlSecAssert2(cert != NULL, NULL);
-
-    /* get ski size */
-    err = gnutls_x509_crt_get_subject_key_id(cert, NULL, &bufSize, &critical);
-    if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecGnuTLSError("gnutls_x509_crt_get_subject_key_id", err, NULL);
-        return(NULL);
-    }
-
-    /* allocate buffer */
-    buf = (xmlSecByte *)xmlMalloc(bufSize + 1);
-    if(buf == NULL) {
-        xmlSecMallocError(bufSize + 1, NULL);
-        return(NULL);
-    }
-
-    /* write it out */
-    err = gnutls_x509_crt_get_subject_key_id(cert, buf, &bufSize, &critical);
-    if(err != GNUTLS_E_SUCCESS) {
-        xmlSecGnuTLSError("gnutls_x509_crt_get_subject_key_id", err, NULL);
-        xmlFree(buf);
-        return(NULL);
-    }
-
-    /* convert to string */
-    res = xmlSecBase64Encode(buf, bufSize, 0);
-    if(res == NULL) {
-        xmlSecInternalError("xmlSecBase64Encode", NULL);
-        xmlFree(buf);
-        return(NULL);
-    }
-
-    /* done */
-    xmlFree(buf);
-    return(res);
-}
-
-
-gnutls_x509_crt_t
-xmlSecGnuTLSX509CertBase64DerRead(xmlChar* buf) {
-    int ret;
-
-    xmlSecAssert2(buf != NULL, NULL);
-
-    /* usual trick with base64 decoding "in-place" */
-    ret = xmlSecBase64Decode(buf, (xmlSecByte*)buf, xmlStrlen(buf));
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecBase64Decode", NULL);
-        return(NULL);
-    }
-
-    return(xmlSecGnuTLSX509CertRead((const xmlSecByte*)buf, ret, xmlSecKeyDataFormatCertDer));
-}
-
 gnutls_x509_crt_t
 xmlSecGnuTLSX509CertRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFormat format) {
     gnutls_x509_crt_t cert = NULL;
     gnutls_x509_crt_fmt_t fmt;
     gnutls_datum_t data;
+    unsigned int bufLen;
     int err;
 
     xmlSecAssert2(buf != NULL, NULL);
@@ -385,9 +334,10 @@ xmlSecGnuTLSX509CertRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFo
         break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
-                         "format=%d", (int)format);
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
         return(NULL);
     }
+    XMLSEC_SAFE_CAST_SIZE_TO_UINT(size, bufLen, return(NULL), NULL);
 
     /* read cert */
     err = gnutls_x509_crt_init(&cert);
@@ -397,7 +347,8 @@ xmlSecGnuTLSX509CertRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFo
     }
 
     data.data = (unsigned char*)buf;
-    data.size = size;
+    data.size = bufLen;
+
     err = gnutls_x509_crt_import(cert, &data, fmt);
     if(err != GNUTLS_E_SUCCESS) {
         xmlSecGnuTLSError("gnutls_x509_crt_import", err, NULL);
@@ -408,48 +359,44 @@ xmlSecGnuTLSX509CertRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFo
     return(cert);
 }
 
-xmlChar*
-xmlSecGnuTLSX509CertBase64DerWrite(gnutls_x509_crt_t cert, int base64LineWrap) {
-    xmlChar * res = NULL;
-    xmlSecByte* buf = NULL;
-    size_t bufSize = 0;
+int
+xmlSecGnuTLSX509CertDerWrite(gnutls_x509_crt_t cert, xmlSecBufferPtr buf) {
+    size_t bufSizeT = 0;
+    xmlSecSize bufSize;
+    xmlSecByte * bufData;
+    int ret;
     int err;
 
-    xmlSecAssert2(cert != NULL, NULL);
+    xmlSecAssert2(cert != NULL, -1);
+    xmlSecAssert2(buf != NULL, -1);
 
     /* get size */
-    err = gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, NULL, &bufSize);
-    if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
+    err = gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, NULL, &bufSizeT);
+    if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSizeT <= 0)) {
         xmlSecGnuTLSError("gnutls_x509_crt_export(GNUTLS_X509_FMT_DER)", err, NULL);
-        return(NULL);
+        return(-1);
     }
+    XMLSEC_SAFE_CAST_SIZE_T_TO_SIZE(bufSizeT, bufSize, return(-1), NULL);
 
     /* allocate buffer */
-    buf = (xmlSecByte *)xmlMalloc(bufSize + 1);
-    if(buf == NULL) {
-        xmlSecMallocError(bufSize + 1, NULL);
-        return(NULL);
+    ret = xmlSecBufferSetSize(buf, bufSize);
+    if(ret < 0) {
+        xmlSecInternalError2("xmlSecBufferSetSize", NULL,
+            "bufSize=" XMLSEC_SIZE_FMT, bufSize);
+        return(-1);
     }
+    bufData = xmlSecBufferGetData(buf);
+    xmlSecAssert2(bufData != NULL, -1);
 
     /* write it out */
-    err = gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, buf, &bufSize);
+    err = gnutls_x509_crt_export(cert, GNUTLS_X509_FMT_DER, bufData, &bufSizeT);
     if(err != GNUTLS_E_SUCCESS) {
         xmlSecGnuTLSError("gnutls_x509_crt_export(GNUTLS_X509_FMT_DER)", err, NULL);
-        xmlFree(buf);
-        return(NULL);
+        return(-1);
     }
 
-    /* convert to string */
-    res = xmlSecBase64Encode(buf, bufSize, base64LineWrap);
-    if(res == NULL) {
-        xmlSecInternalError("xmlSecBase64Encode", NULL);
-        xmlFree(buf);
-        return(NULL);
-    }
-
-    /* done */
-    xmlFree(buf);
-    return(res);
+    /* success */
+    return(0);
 }
 
 void
@@ -522,30 +469,39 @@ xmlSecGnuTLSX509CertDebugXmlDump(gnutls_x509_crt_t cert, FILE* output) {
  *
  ************************************************************************/
 
-/* HACK: gnutls doesn't have crl duplicate function, so we simply 
+/* HACK: gnutls doesn't have crl duplicate function, so we simply
  write crl out and then read it back */
 gnutls_x509_crl_t
 xmlSecGnuTLSX509CrlDup(gnutls_x509_crl_t src) {
-    xmlChar * buf = NULL;
+    xmlSecBuffer buf;
     gnutls_x509_crl_t res = NULL;
+    int ret;
 
     xmlSecAssert2(src != NULL, NULL);
 
-    buf = xmlSecGnuTLSX509CrlBase64DerWrite(src, 0);
-    if(buf == NULL) {
-        xmlSecInternalError("xmlSecGnuTLSX509CrlBase64DerWrite", NULL);
+    ret = xmlSecBufferInitialize(&buf, 0);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecBufferInitialize", NULL);
         return (NULL);
     }
 
-    res = xmlSecGnuTLSX509CrlBase64DerRead(buf);
+    ret = xmlSecGnuTLSX509CrlDerWrite(src, &buf);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGnuTLSX509CrlDerWrite", NULL);
+        xmlSecBufferFinalize(&buf);
+        return (NULL);
+    }
+
+    res = xmlSecGnuTLSX509CrlRead(xmlSecBufferGetData(&buf), xmlSecBufferGetSize(&buf),
+        xmlSecKeyDataFormatCertDer);
     if(res == NULL) {
-        xmlSecInternalError("xmlSecGnuTLSX509CrlBase64DerRead", NULL);
-        xmlFree(buf);
+        xmlSecInternalError("xmlSecGnuTLSX509CrlRead", NULL);
+        xmlSecBufferFinalize(&buf);
         return (NULL);
     }
 
     /* done */
-    xmlFree(buf);
+    xmlSecBufferFinalize(&buf);
     return (res);
 }
 
@@ -583,27 +539,12 @@ xmlSecGnuTLSX509CrlGetIssuerDN(gnutls_x509_crl_t crl) {
     return(BAD_CAST buf);
 }
 
-gnutls_x509_crl_t
-xmlSecGnuTLSX509CrlBase64DerRead(xmlChar* buf) {
-    int ret;
-
-    xmlSecAssert2(buf != NULL, NULL);
-
-    /* usual trick with base64 decoding "in-place" */
-    ret = xmlSecBase64Decode(buf, (xmlSecByte*)buf, xmlStrlen(buf));
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecBase64Decode", NULL);
-        return(NULL);
-    }
-
-    return(xmlSecGnuTLSX509CrlRead((const xmlSecByte*)buf, ret, xmlSecKeyDataFormatCertDer));
-}
-
 gnutls_x509_crl_t
 xmlSecGnuTLSX509CrlRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFormat format) {
     gnutls_x509_crl_t crl = NULL;
     gnutls_x509_crt_fmt_t fmt;
     gnutls_datum_t data;
+    unsigned int bufLen;
     int err;
 
     xmlSecAssert2(buf != NULL, NULL);
@@ -621,9 +562,10 @@ xmlSecGnuTLSX509CrlRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFor
         break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
-                         "format=%d", (int)format);
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
         return(NULL);
     }
+    XMLSEC_SAFE_CAST_SIZE_TO_UINT(size, bufLen, return(NULL), NULL);
 
     /* read crl */
     err = gnutls_x509_crl_init(&crl);
@@ -633,7 +575,7 @@ xmlSecGnuTLSX509CrlRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFor
     }
 
     data.data = (unsigned char*)buf;
-    data.size = size;
+    data.size = bufLen;
     err = gnutls_x509_crl_import(crl, &data, fmt);
     if(err != GNUTLS_E_SUCCESS) {
         xmlSecGnuTLSError("gnutls_x509_crl_import", err, NULL);
@@ -644,48 +586,44 @@ xmlSecGnuTLSX509CrlRead(const xmlSecByte* buf, xmlSecSize size, xmlSecKeyDataFor
     return(crl);
 }
 
-xmlChar*
-xmlSecGnuTLSX509CrlBase64DerWrite(gnutls_x509_crl_t crl, int base64LineWrap) {
-    xmlChar * res = NULL;
-    xmlSecByte* buf = NULL;
-    size_t bufSize = 0;
+int
+xmlSecGnuTLSX509CrlDerWrite(gnutls_x509_crl_t crl, xmlSecBufferPtr buf) {
+    size_t bufSizeT = 0;
+    xmlSecSize bufSize;
+    xmlSecByte * bufData;
+    int ret;
     int err;
 
-    xmlSecAssert2(crl != NULL, NULL);
+    xmlSecAssert2(crl != NULL, -1);
+    xmlSecAssert2(buf != NULL, -1);
 
     /* get size */
-    err = gnutls_x509_crl_export(crl, GNUTLS_X509_FMT_DER, NULL, &bufSize);
-    if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSize <= 0)) {
-        xmlSecGnuTLSError("gnutls_x509_crl_export(GNUTLS_X509_FMT_DER)", err, NULL);
-        return(NULL);
+    err = gnutls_x509_crl_export(crl, GNUTLS_X509_FMT_DER, NULL, &bufSizeT);
+    if((err != GNUTLS_E_SHORT_MEMORY_BUFFER) || (bufSizeT <= 0)) {
+        xmlSecGnuTLSError("ggnutls_x509_crl_export(GNUTLS_X509_FMT_DER)", err, NULL);
+        return(-1);
     }
+    XMLSEC_SAFE_CAST_SIZE_T_TO_SIZE(bufSizeT, bufSize, return(-1), NULL);
 
     /* allocate buffer */
-    buf = (xmlSecByte *)xmlMalloc(bufSize + 1);
-    if(buf == NULL) {
-        xmlSecMallocError(bufSize + 1, NULL);
-        return(NULL);
+    ret = xmlSecBufferSetSize(buf, bufSize);
+    if(ret < 0) {
+        xmlSecInternalError2("xmlSecBufferSetSize", NULL,
+            "bufSize=" XMLSEC_SIZE_FMT, bufSize);
+        return(-1);
     }
+    bufData = xmlSecBufferGetData(buf);
+    xmlSecAssert2(bufData != NULL, -1);
 
     /* write it out */
-    err = gnutls_x509_crl_export(crl, GNUTLS_X509_FMT_DER, buf, &bufSize);
+    err = gnutls_x509_crl_export(crl,GNUTLS_X509_FMT_DER, bufData, &bufSizeT);
     if(err != GNUTLS_E_SUCCESS) {
-        xmlSecGnuTLSError("gnutls_x509_crl_export(GNUTLS_X509_FMT_DER)", err, NULL);
-        xmlFree(buf);
-        return(NULL);
+        xmlSecGnuTLSError("ggnutls_x509_crl_export(GNUTLS_X509_FMT_DER)", err, NULL);
+        return(-1);
     }
 
-    /* convert to string */
-    res = xmlSecBase64Encode(buf, bufSize, base64LineWrap);
-    if(res == NULL) {
-        xmlSecInternalError("xmlSecBase64Encode", NULL);
-        xmlFree(buf);
-        return(NULL);
-    }
-
-    /* done */
-    xmlFree(buf);
-    return(res);
+    /* success */
+    return(0);
 }
 
 void
@@ -725,10 +663,10 @@ xmlSecGnuTLSX509CrlDebugXmlDump(gnutls_x509_crl_t crl, FILE* output) {
  * Misc. utils/helpers
  *
  ************************************************************************/
+#define XMLSEC_GNUTLS_INT_TO_STR_MAX_SIZE 64
 xmlChar*
 xmlSecGnuTLSASN1IntegerWrite(const unsigned char * data, size_t len) {
     xmlChar *res = NULL;
-    int resLen = 64; /* not more than 64 chars */
     unsigned long long int val = 0;
     size_t ii = 0;
     int shift = 0;
@@ -742,13 +680,13 @@ xmlSecGnuTLSASN1IntegerWrite(const unsigned char * data, size_t len) {
         val |= ((unsigned long long)data[ii - 1]) << shift;
     }
 
-    res = (xmlChar*)xmlMalloc(resLen + 1);
+    res = (xmlChar*)xmlMalloc(XMLSEC_GNUTLS_INT_TO_STR_MAX_SIZE + 1);
     if(res == NULL) {
-        xmlSecMallocError(resLen + 1, NULL);
+        xmlSecMallocError(XMLSEC_GNUTLS_INT_TO_STR_MAX_SIZE + 1, NULL);
         return(NULL);
     }
 
-    ret = xmlStrPrintf(res, resLen, "%llu", val);
+    ret = xmlStrPrintf(res, XMLSEC_GNUTLS_INT_TO_STR_MAX_SIZE, "%llu", val);
     if(ret < 0) {
         xmlSecXmlError("xmlStrPrintf", NULL);
         xmlFree(res);
@@ -775,6 +713,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
     gnutls_x509_crt_t cert = NULL;
     gnutls_datum_t datum;
     xmlSecSize certsSize;
+    unsigned int dataLen;
     int res = -1;
     int idx;
     int err;
@@ -788,6 +727,8 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
     xmlSecAssert2((*key_cert) == NULL, -1);
     xmlSecAssert2(certsList != NULL, -1);
 
+    XMLSEC_SAFE_CAST_SIZE_TO_UINT(dataSize, dataLen, return(-1), NULL);
+
     /* read pkcs12 in internal structure */
     err = gnutls_pkcs12_init(&pkcs12);
     if(err != GNUTLS_E_SUCCESS) {
@@ -796,7 +737,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
     }
 
     datum.data = (unsigned char *)data;
-    datum.size = dataSize;
+    datum.size = dataLen;
     err = gnutls_pkcs12_import(pkcs12, &datum, GNUTLS_X509_FMT_DER, 0);
     if(err != GNUTLS_E_SUCCESS) {
         xmlSecGnuTLSError("gnutls_pkcs12_import", err, NULL);
@@ -814,7 +755,7 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
     for(idx = 0; ; ++idx) {
         int bag_type;
         int elements_in_bag;
-        int ii;
+        unsigned int num, ii;
 
         err = gnutls_pkcs12_bag_init(&bag);
         if(err != GNUTLS_E_SUCCESS) {
@@ -851,7 +792,8 @@ xmlSecGnuTLSPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
             xmlSecGnuTLSError("gnutls_pkcs12_bag_get_count", elements_in_bag, NULL);
             goto done;
         }
-        for(ii = 0; ii < elements_in_bag; ++ii) {
+        XMLSEC_SAFE_CAST_INT_TO_UINT(elements_in_bag, num, goto done, NULL);
+        for(ii = 0; ii < num; ++ii) {
             bag_type = gnutls_pkcs12_bag_get_type(bag, ii);
             if(bag_type < 0) {
                 xmlSecGnuTLSError("gnutls_pkcs12_bag_get_type", bag_type, NULL);
@@ -1114,7 +1056,7 @@ xmlSecGnuTLSDnAttrrsFind(const xmlSecGnuTLSDnAttr * attrs,
 
 int
 xmlSecGnuTLSDnAttrsEqual(const xmlSecGnuTLSDnAttr * ll, xmlSecSize llSize,
-                         const xmlSecGnuTLSDnAttr * rr, xmlSecSize rrSize) 
+                         const xmlSecGnuTLSDnAttr * rr, xmlSecSize rrSize)
 {
     xmlSecSize llNum = 0;
     xmlSecSize rrNum = 0;
@@ -1164,7 +1106,7 @@ xmlSecGnuTLSDnAttrsEqual(const xmlSecGnuTLSDnAttr * ll, xmlSecSize llSize,
 /*
 Distinguished name syntax
 
-The formal syntax for a Distinguished Name (DN) is based on RFC 2253. 
+The formal syntax for a Distinguished Name (DN) is based on RFC 2253.
 The Backus Naur Form (BNF) syntax is defined as follows:
 
     <name> ::= <name-component> ( <spaced-separator> )
@@ -1214,7 +1156,7 @@ White-space characters (spaces) might be present on either side of the comma or
 semicolon. The white-space characters are ignored, and the semicolon is replaced
 with a comma.
 
-In addition, space (' ' ASCII 32) characters may be present either before or 
+In addition, space (' ' ASCII 32) characters may be present either before or
 after a '+' or '='. These space characters are ignored when parsing.
 */
 enum xmlSecGnuTLSDnParseState {
@@ -1238,7 +1180,7 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
     xmlChar ch;
     enum xmlSecGnuTLSDnParseState state;
     int slash;
-    xmlSecSize pos;
+    xmlSecSize size, pos;
     int res = -1;
 
     xmlSecAssert2(dn != NULL, -1);
@@ -1246,9 +1188,10 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
     xmlSecAssert2(attrsSize > 0, -1);
 
     /* allocate buffer, we don't need more than string */
-    tmp = (xmlChar *)xmlMalloc(xmlStrlen(dn) + 1);
+    size = xmlSecStrlen(dn);
+    tmp = (xmlChar *)xmlMalloc(size + 1);
     if(tmp == NULL) {
-        xmlSecMallocError(xmlStrlen(dn) + 1, NULL);
+        xmlSecMallocError(size + 1, NULL);
         goto done;
     }
 
@@ -1302,7 +1245,7 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
                 if(ch != '\"') {
                     state = xmlSecGnuTLSDnParseState_String;
                     slash = 0;
-                    --dn; /* small hack, so we can look at the same char 
+                    --dn; /* small hack, so we can look at the same char
                            again with the correct state */
                 } else {
                     state = xmlSecGnuTLSDnParseState_QuotedString;
@@ -1344,7 +1287,7 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
             } else if(ch == '\\') {
                 slash = 1;
             } else if(ch == '\"') {
-                *(p) = '\0'; 
+                *(p) = '\0';
                 /* don't remove spaces for quoted string */
 
                 attrs[pos].value = xmlStrdup(tmp);
@@ -1381,21 +1324,10 @@ xmlSecGnuTLSDnAttrsParse(const xmlChar * dn,
 
     /* check end state */
     if(state != xmlSecGnuTLSDnParseState_BeforeNameComponent) {
-        xmlSecInvalidIntegerDataError("state", state, "xmlSecGnuTLSDnParseState_BeforeNameComponent", NULL);
+        xmlSecUnsupportedEnumValueError("state", state, NULL);
         goto done;
     }
 
-    /* debug
-    {
-        xmlSecSize ii;
-        for(ii = 0; ii < attrsSize; ++ii) {
-            if(attrs[ii].key != NULL) {
-                printf("DEBUG: attrs - %s=>%s\n", attrs[ii].key, attrs[ii].value);
-            }
-        }
-    }
-    */
-
     /* done */
     res = 0;
 
