@@ -32,6 +32,8 @@
 #include <xmlsec/mscng/crypto.h>
 #include <xmlsec/mscng/certkeys.h>
 
+#include "../cast_helpers.h"
+
 /**************************************************************************
  *
  * Internal MSCng signatures ctx
@@ -54,13 +56,9 @@ struct _xmlSecMSCngSignatureCtx {
  *
  * Signature transforms
  *
- * xmlSecMSCngSignatureCtx is located after xmlSecTransform
- *
  *****************************************************************************/
-#define xmlSecMSCngSignatureSize     \
-    (sizeof(xmlSecTransform) + sizeof(xmlSecMSCngSignatureCtx))
-#define xmlSecMSCngSignatureGetCtx(transform) \
-    ((xmlSecMSCngSignatureCtxPtr)(((xmlSecByte*)(transform)) + sizeof(xmlSecTransform)))
+XMLSEC_TRANSFORM_DECLARE(MSCngSignature, xmlSecMSCngSignatureCtx)
+#define xmlSecMSCngSignatureSize XMLSEC_TRANSFORM_SIZE(MSCngSignature)
 
 static int      xmlSecMSCngSignatureCheckId             (xmlSecTransformPtr transform);
 static int      xmlSecMSCngSignatureInitialize          (xmlSecTransformPtr transform);
@@ -138,6 +136,12 @@ static int xmlSecMSCngSignatureCheckId(xmlSecTransformPtr transform) {
     } else
 #endif /* XMLSEC_NO_SHA256 */
 
+#ifndef XMLSEC_NO_SHA384
+    if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha384Id)) {
+       return(1);
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
 #ifndef XMLSEC_NO_SHA512
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha512Id)) {
        return(1);
@@ -227,6 +231,13 @@ static int xmlSecMSCngSignatureInitialize(xmlSecTransformPtr transform) {
     } else
 #endif /* XMLSEC_NO_SHA256 */
 
+#ifndef XMLSEC_NO_SHA384
+    if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha384Id)) {
+        ctx->pszHashAlgId = BCRYPT_SHA384_ALGORITHM;
+        ctx->keyId = xmlSecMSCngKeyDataEcdsaId;
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
 #ifndef XMLSEC_NO_SHA512
     if(xmlSecTransformCheckId(transform, xmlSecMSCngTransformEcdsaSha512Id)) {
         ctx->pszHashAlgId = BCRYPT_SHA512_ALGORITHM;
@@ -258,20 +269,28 @@ static void xmlSecMSCngSignatureFinalize(xmlSecTransformPtr transform) {
         xmlSecKeyDataDestroy(ctx->data);
     }
 
-    if(ctx->pbHash != NULL) {
-        xmlFree(ctx->pbHash);
-    }
+    // MSDN documents at
+    // https://msdn.microsoft.com/en-us/library/windows/desktop/aa376217(v=vs.85).aspx
+    // that the order of cleanup should be:
+    // - algo handle
+    // - hash handle
+    // - hash object pointer
+    // - hash pointer
 
     if(ctx->hHashAlg != 0) {
         BCryptCloseAlgorithmProvider(ctx->hHashAlg, 0);
     }
 
+    if(ctx->hHash != 0) {
+        BCryptDestroyHash(ctx->hHash);
+    }
+
     if(ctx->pbHashObject != NULL) {
         xmlFree(ctx->pbHashObject);
     }
 
-    if(ctx->hHash != 0) {
-        BCryptDestroyHash(ctx->hHash);
+    if(ctx->pbHash != NULL) {
+        xmlFree(ctx->pbHash);
     }
 
     memset(ctx, 0, sizeof(xmlSecMSCngSignatureCtx));
@@ -337,7 +356,7 @@ static int xmlSecMSCngSignatureVerify(xmlSecTransformPtr transform,
     NTSTATUS status;
     BCRYPT_PKCS1_PADDING_INFO info;
     BCRYPT_PKCS1_PADDING_INFO* pInfo = NULL;
-    DWORD infoFlags = 0;
+    DWORD dwDataSize, infoFlags = 0;
 
     xmlSecAssert2(xmlSecMSCngSignatureCheckId(transform), -1);
     xmlSecAssert2(transform->operation == xmlSecTransformOperationVerify, -1);
@@ -352,8 +371,7 @@ static int xmlSecMSCngSignatureVerify(xmlSecTransformPtr transform,
 
     pubkey = xmlSecMSCngKeyDataGetPubKey(ctx->data);
     if(pubkey == 0) {
-        xmlSecInternalError("xmlSecMSCngKeyDataGetPubKey",
-            xmlSecTransformGetName(transform));
+        xmlSecInternalError("xmlSecMSCngKeyDataGetPubKey", xmlSecTransformGetName(transform));
         return(-1);
     }
 
@@ -365,13 +383,14 @@ static int xmlSecMSCngSignatureVerify(xmlSecTransformPtr transform,
         infoFlags = BCRYPT_PAD_PKCS1;
     }
 
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(dataSize, dwDataSize, return(-1), xmlSecTransformGetName(transform));
     status = BCryptVerifySignature(
         pubkey,
         pInfo,
         ctx->pbHash,
         ctx->cbHash,
         (PBYTE)data,
-        dataSize,
+        dwDataSize,
         infoFlags);
     if(status != STATUS_SUCCESS) {
         if(status == STATUS_INVALID_SIGNATURE) {
@@ -493,24 +512,25 @@ xmlSecMSCngSignatureExecute(xmlSecTransformPtr transform, int last, xmlSecTransf
 
     if(transform->status == xmlSecTransformStatusWorking) {
         if(inSize > 0) {
+            DWORD dwInSize;
+
             xmlSecAssert2(outSize == 0, -1);
 
             /* hash some data */
+            XMLSEC_SAFE_CAST_SIZE_TO_ULONG(inSize, dwInSize, return(-1), xmlSecTransformGetName(transform));
             status = BCryptHashData(
                 ctx->hHash,
                 (PBYTE)xmlSecBufferGetData(&transform->inBuf),
-                inSize,
+                dwInSize,
                 0);
             if(status != STATUS_SUCCESS) {
-                xmlSecMSCngNtError("BCryptHashData",
-                    xmlSecTransformGetName(transform), status);
+                xmlSecMSCngNtError("BCryptHashData", xmlSecTransformGetName(transform), status);
                 return(-1);
             }
 
             ret = xmlSecBufferRemoveHead(&transform->inBuf, inSize);
             if(ret < 0) {
-                xmlSecInternalError("xmlSecBufferRemoveHead",
-                                     xmlSecTransformGetName(transform));
+                xmlSecInternalError("xmlSecBufferRemoveHead", xmlSecTransformGetName(transform));
                 return(-1);
             }
         }
@@ -535,8 +555,7 @@ xmlSecMSCngSignatureExecute(xmlSecTransformPtr transform, int last, xmlSecTransf
 
                 privkey = xmlSecMSCngKeyDataGetPrivKey(ctx->data);
                 if(privkey == 0) {
-                    xmlSecInternalError("xmlSecMSCngKeyDataGetPrivKey",
-                        xmlSecTransformGetName(transform));
+                    xmlSecInternalError("xmlSecMSCngKeyDataGetPrivKey", xmlSecTransformGetName(transform));
                     return(-1);
                 }
 
@@ -551,26 +570,25 @@ xmlSecMSCngSignatureExecute(xmlSecTransformPtr transform, int last, xmlSecTransf
                     &cbSignature,
                     0);
                 if(status != STATUS_SUCCESS) {
-                    xmlSecMSCngNtError("NCryptSignHash",
-                        xmlSecTransformGetName(transform), status);
+                    xmlSecMSCngNtError("NCryptSignHash", xmlSecTransformGetName(transform), status);
                     return(-1);
                 }
-                outSize = (xmlSecSize)cbSignature;
+                XMLSEC_SAFE_CAST_ULONG_TO_SIZE(cbSignature, outSize, return(-1), xmlSecTransformGetName(transform));
 
                 /* allocate the signature buffer on the heap */
                 ret = xmlSecBufferSetSize(&transform->outBuf, outSize);
                 if(ret < 0) {
-                    xmlSecInternalError2("xmlSecBufferSetSize",
-                        xmlSecTransformGetName(transform), "size=%d", outSize);
+                    xmlSecInternalError2("xmlSecBufferSetSize", xmlSecTransformGetName(transform),
+                            "size=" XMLSEC_SIZE_FMT, outSize);
                     return(-1);
                 }
 
                 /* sign the hash */
-		if(ctx->keyId == xmlSecMSCngKeyDataRsaId) {
+        if(ctx->keyId == xmlSecMSCngKeyDataRsaId) {
                     info.pszAlgId = ctx->pszHashAlgId;
                     pInfo = &info;
                     infoFlags = BCRYPT_PAD_PKCS1;
-		}
+        }
                 status = NCryptSignHash(
                     privkey,
                     pInfo,
@@ -979,6 +997,52 @@ xmlSecMSCngTransformEcdsaSha256GetKlass(void) {
 }
 #endif /* XMLSEC_NO_SHA256 */
 
+#ifndef XMLSEC_NO_SHA384
+/****************************************************************************
+ *
+ * ECDSA-SHA384 signature transform
+ *
+ ***************************************************************************/
+static xmlSecTransformKlass xmlSecMSCngEcdsaSha384Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),              /* xmlSecSize klassSize */
+    xmlSecMSCngSignatureSize,                  /* xmlSecSize objSize */
+
+    xmlSecNameEcdsaSha384,                     /* const xmlChar* name; */
+    xmlSecHrefEcdsaSha384,                     /* const xmlChar* href; */
+    xmlSecTransformUsageSignatureMethod,       /* xmlSecTransformUsage usage; */
+
+    xmlSecMSCngSignatureInitialize,            /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecMSCngSignatureFinalize,              /* xmlSecTransformFinalizeMethod finalize; */
+    NULL,                                      /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                      /* xmlSecTransformNodeWriteMethod writeNode; */
+    xmlSecMSCngSignatureSetKeyReq,             /* xmlSecTransformSetKeyReqMethod setKeyReq; */
+    xmlSecMSCngSignatureSetKey,                /* xmlSecTransformSetKeyMethod setKey; */
+    xmlSecMSCngSignatureVerify,                /* xmlSecTransformVerifyMethod verify; */
+    xmlSecTransformDefaultGetDataType,         /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,             /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,              /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                      /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                      /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecMSCngSignatureExecute,               /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                      /* void* reserved0; */
+    NULL,                                      /* void* reserved1; */
+};
+
+/**
+ * xmlSecMSCngTransformEcdsaSha384GetKlass:
+ *
+ * The ECDSA-SHA384 signature transform klass.
+ *
+ * Returns: ECDSA-SHA384 signature transform klass.
+ */
+xmlSecTransformId
+xmlSecMSCngTransformEcdsaSha384GetKlass(void) {
+    return(&xmlSecMSCngEcdsaSha384Klass);
+}
+#endif /* XMLSEC_NO_SHA384 */
+
 #ifndef XMLSEC_NO_SHA512
 /****************************************************************************
  *
